/*----------------------------------------------------------------------*/
/*  Omatrix - 3D orthogonal matrices (orientation matrices) 
        - methods:
            Rotation - gives a rotation matrix in a coordinate plane
            Times - does matrix multiplication
            Equal - sets M = another Omatrix
            Tranpose - returns the tranpose (which is the inverse here)
            FindTran - given 2 matrices it finds the angle of rotation 
                between them and also the plane of rotation 
            Assign - 9 floats
                                                                        */
/*----------------------------------------------------------------------*/
/* Jim Morey -- morey@math.ubc.ca -- January 24, 1996                   */
/*----------------------------------------------------------------------*/

public class Omatrix extends Object{
  public double[][] M;
  private int row,col,k,l;
 
  Omatrix(){
    M = new double[3][3];

    Assign(1,0,0,0,1,0,0,0,1);
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public void Rotation (int i, int j, double angle) {
    for (row=0; row < 3; row++) {
      for (col=0; col <3; col++) {
        if (row != col) { M[row][col] = 0.0; }
        else { M[row][col] = 1.0; }
      }
    }
    M[i][i] = Math.cos(angle);
    M[j][j] = Math.cos(angle);
    M[i][j] = Math.sin(angle);
    M[j][i] = -Math.sin(angle);
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public Omatrix Times(Omatrix N){
    Omatrix temp = new Omatrix();
    for (row=0; row < 3; row++) {
      for (col=0; col <3; col++) {
        temp.M[row][col] = 0.0;
        for (k=0; k < 3; k++) {
          temp.M[row][col] += M[row][k] * N.M[k][col];
        }
      }
    }

    return temp;
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public void Equal(Omatrix N){
    for (row=0; row < 3; row++) {
      for (col=0; col <3; col++) {
        M[row][col] = N.M[row][col];
      }
    }
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public Omatrix Transpose(){
    Omatrix temp = new Omatrix();
    for (row=0; row < 3; row++) {
      for (col=0; col <3; col++) {
        temp.M[row][col]=M[col][row];
      }
    }
    return temp;
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  private void ExRow(int i, int j){
    double swap;
    for (col=0; col <3; col++) {
      swap = M[i][col];
      M[i][col] = M[j][col];
      M[j][col] = swap;
    }
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  private void RowAdd(int i, double s1, int j, double s2){
    for (col=0; col <3; col++) 
      M[i][col] = M[i][col]*s1 + M[j][col]*s2;
  }
    
  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public double FindTran(Omatrix A, Omatrix B){
    double dot,diff,vec[];
    Omatrix C = new Omatrix();

    Assign(1,0,0,0,1,0,0,0,1);
    C = B.Times(A.Transpose()); 
    /* .. first solve the eigenvector of C -- the axis of rotation .. */
    /* .. this probably is not the best way to do it ... */
    /* .. and it's sloppy .. */

    C.M[0][0] -= 1.0;
    C.M[1][1] -= 1.0;
    C.M[2][2] -= 1.0;
    if (Math.abs(C.M[0][0])+Math.abs(C.M[1][0])+Math.abs(C.M[2][0]) > 0.000001){
      if (Math.abs(C.M[0][1])+Math.abs(C.M[1][1])+
          Math.abs(C.M[2][1]) < 0.000001){
        ExRow(0,2);
        ExRow(1,2);
      } else{ 
        /* .. find the cross product of C.M[*][0] & C.M[*][1] .. */
        M[0][0] = C.M[1][0]*C.M[2][1] - C.M[2][0]*C.M[1][1];
        M[1][0] = C.M[2][0]*C.M[0][1] - C.M[0][0]*C.M[2][1];
        M[2][0] = C.M[0][0]*C.M[1][1] - C.M[1][0]*C.M[0][1];
        if (Math.abs(M[0][0])+Math.abs(M[1][0])+Math.abs(M[2][0]) < 0.01){
          /* .. C.M[*][0] & C.M[*][1] are not really linearly independant .. */
          M[0][0] = C.M[1][2]*C.M[2][1] - C.M[2][2]*C.M[1][1];
          M[1][0] = C.M[2][2]*C.M[0][1] - C.M[0][2]*C.M[2][1];
          M[2][0] = C.M[0][2]*C.M[1][1] - C.M[1][2]*C.M[0][1];
          for (row=0;row < 3;row++) {
            M[row][1] = C.M[row][2];   
            M[row][2] = C.M[row][1];   
          }
        } else{
          for (row=0;row < 3;row++) {
            M[row][1] = C.M[row][0];   
            M[row][2] = C.M[row][1];   
          }
        }
        /* .. orthonormalize M */  
        for (col =0;col<3;col++){
          dot = Math.sqrt(M[0][col]*M[0][col]+M[1][col]*M[1][col]+
            M[2][col]*M[2][col]);
          if (dot == 0) System.out.println("ortho Fail!");
          else{
              M[0][col] /= dot;
              M[1][col] /= dot;
              M[2][col] /= dot;
          }
          for (k=col+1; k <3; k++) {
            dot = M[0][col]*M[0][k] + M[1][col]*M[1][k] + M[2][col]*M[2][k];
            M[0][k] -= dot*M[0][col];
            M[1][k] -= dot*M[1][col];
            M[2][k] -= dot*M[2][col];
          }
        }
      }
    } 
    C.M[0][0] += 1.0;
    C.M[1][1] += 1.0;
    C.M[2][2] += 1.0;
    vec = new double[3];
    vec[0] = M[0][1]*C.M[0][0] + M[1][1]*C.M[0][1] + M[2][1]*C.M[0][2];
    vec[1] = M[0][1]*C.M[1][0] + M[1][1]*C.M[1][1] + M[2][1]*C.M[1][2];
    vec[2] = M[0][1]*C.M[2][0] + M[1][1]*C.M[2][1] + M[2][1]*C.M[2][2];

    dot = Math.acos(vec[0]*M[0][1] + vec[1]*M[1][1] + vec[2]*M[2][1]);
 
    Omatrix tmp= new Omatrix();
    tmp.Rotation(1,2,dot);
    tmp = Times(tmp.Times(Transpose()));
    diff = 0;
    for (row=0; row < 3; row++)
      for (col=0; col <3; col++) 
        diff += Math.abs(tmp.M[row][col] - C.M[row][col]);
    if (diff >0.01) dot = -dot;    

    return dot;
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public void Assign(double a, double b, double c, 
                        double d, double e, double f,
                        double g, double h, double i ){
    M[0][0] = a;
    M[0][1] = b;
    M[0][2] = c;
    M[1][0] = d;
    M[1][1] = e;
    M[1][2] = f;
    M[2][0] = g;
    M[2][1] = h;
    M[2][2] = i;
  }

  /*  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
  public String toString() {
    String              str;
 
    str = "Omatrix ";
    for (row=0;row<3;row++){ 
      str = str.concat(row+">>(");
      for (col=0;col<3;col++) str = str.concat(" "+M[row][col]);
      str = str.concat(")\n        ");
    }
 
    return str;
  }
}
