import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;

/**
 * A class that work with file
 *
 * @version 	1.0 03/24/96
 * @author 	    Kang, Dae Woong (namu@star.elim.net)
 */
class FileFrame extends Frame
{
	Applet applet;
	WorkPanel work;
	String drawIndexName = new String("index.draw");

	static final String OK = "OK";
	static final String CANCEL = "CANCEL";

	TextField tfInput = new TextField(20);
	Button btnOK = new Button(OK);
	Button btnCancel = new Button(CANCEL);
	List lstFile = new List(10, false);
	URL urlDocument = null;

	String imageDir;
	String drawDir;

	int mode = 0;
	final int REF_IMAGE = 1;
	final int REF_DRAW = 2;
	final int READ_DRAW = 3;
	final int WRITE_DRAW = 4;


	String fileName = null;
	String dir = null;

	
    public FileFrame(Applet applet, WorkPanel work, URL urlDocument)
	{
		super();
		this.applet = applet;
		this.work = work;

		this.urlDocument = urlDocument;

		imageDir = applet.getParameter("image");
		if (imageDir == null)
			imageDir = "";
		drawDir = applet.getParameter("draw");
		if (drawDir == null)
			drawDir = "";

		Panel pnl = new Panel();

		pnl.setLayout(new FlowLayoutCust(FlowLayoutCust.HORIZONTAL));
		pnl.add(tfInput);
		pnl.add(btnOK);
		pnl.add(btnCancel);

		setLayout(new BorderLayout());
		add("Center", lstFile);
		add("South", pnl);
    }

	private synchronized boolean readIndex(String nmFile)
	{

		URL  url = null;
		try
		{
			url =  new URL(urlDocument, nmFile);
		}
		catch (MalformedURLException e)
		{
			System.err.println(nmFile + " document not found" + e);
			return false;
		}

		try
		{
			DataInputStream dis= new DataInputStream(url.openStream());

			String buf;
			while (null != (buf = dis.readLine()))
				lstFile.addItem(buf);
		}
		catch (IOException e) 
		{
            System.err.println(e);
            return false;
        }

		return true;
	}
	 
    public boolean handleEvent(Event e) 
	{
        switch (e.id)
		{
		case Event.LIST_SELECT:
			fileName = lstFile.getSelectedItem();
			if (mode == WRITE_DRAW)
				tfInput.setText(fileName);
        }
        return super.handleEvent(e);
    }

	public String getPath()
	{
		return dir + fileName;
	}

	public boolean action(Event e, Object target)
	{
		if (e.target instanceof Button)
		{
			if (e.arg == OK)
			{
				applet.enable();
				hide();
				switch (mode)
				{
				case READ_DRAW:
					work.readDraw(getPath());
					break;
				case WRITE_DRAW:
					work.writeDraw(dir, tfInput.getText());
					break;
				}
				return true;
			}
			else if (e.arg == CANCEL)
			{
				applet.enable();
				hide();
				return true;
			}
		}
		return false;
	}

	public void refImage()
	{
		mode = REF_IMAGE;
		fileName = null;
		setTitle("Image Select");
		dir = imageDir;
		btnOK.hide();
		btnCancel.hide();
		tfInput.hide();
		layout();
		lstFile.delItems(0, lstFile.countItems() - 1);
		show();

		readIndex(dir + "index.image");
		if (lstFile.countItems() > 0)
		{
			lstFile.select(0);
		}
	}

	public void refDraw()
	{
		mode = REF_DRAW;
		fileName = null;
		setTitle("Draw Select");
		btnOK.hide();
		btnCancel.hide();
		tfInput.hide();
		layout();
		lstFile.delItems(0, lstFile.countItems() - 1);
		show();
		dir = drawDir;
		readIndex(dir + drawIndexName);
		if (lstFile.countItems() > 0)
		{
			lstFile.select(0);
		}
	}

	public void readDraw()
	{
		mode = READ_DRAW;
		fileName = null;
		setTitle("Read Draw");
		dir = drawDir;
		btnOK.show();
		btnCancel.show();
		tfInput.show();
		layout();
		lstFile.delItems(0, lstFile.countItems() - 1);
		show();
		readIndex(dir + drawIndexName);
		applet.disable();
		if (lstFile.countItems() > 0)
		{
			lstFile.select(0);
		}
	}

	public void writeDraw()
	{
		mode = WRITE_DRAW;
		fileName = null;
		setTitle("Write Draw");
		dir = drawDir;
		btnOK.show();
		btnCancel.show();
		tfInput.show();
		layout();
		lstFile.delItems(0, lstFile.countItems() - 1);
		show();
		readIndex(dir + drawIndexName);
		applet.disable();
	}
}
