/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)Divider.java  0.00 01-Jan-96

        A tabbed page in a notebook.

    Authors:

        rphall   Rick Hall

    Version Ident:

        $Header: /PjJavaClient/src/pj/awt/Divider.java 2     1/22/96 9:09a Rphall $

    History:

        0.00 01-Jan-96  rphall      Initial Creation

---------------------------------------------------------------------------*/

package pj.awt;

import pj.awt.Page;
import pj.awt.TabSpec;

/**
 * A Divider is tabbed page in a notebook. The Divider class provides only
 * a specification for the tab that should be associated with this page.
 * The notebook is responsible for implementation and visual presentation
 * of the tab.
 *
 * @see     pj.awt.Notebook
 * @see     pj.awt.TabSpec
 * @version 0.00 01-Jan-96
 * @author  rphall
*/
public class Divider extends Page
    {


    // --- Instance variables

    private TabSpec tabspec;

    // --- Public constructors

    /**
     * Construct a Divider with a given tab specification.
     * @param tab   A specification for the tab that should be
     * associated with this page.
    */
    public Divider(TabSpec tab)
        {
        super(tab.strPageName);
        tabspec = tab;
        }

    // --- Public operations

    /**
     * @param tab   The tab specification for a divider.
    */
    public void setTabSpec(TabSpec tab)
        { 
        tabspec = tab; 
        }

    /**
     * @return The tab specification for a divider.
    */
    public TabSpec getTabSpec()
        {
        return tabspec; 
        }

    
    } // Divider
