/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)PaperView.java  0.00 07-Jan-96

        A view onto one or more sections of a Personal Journal paper.


    Authors:

        rphall   Rick Hall


    Version Ident:

        $Header$


    History:

        0.00 07-Jan-96  rphall      Initial Creation
        0.01 23-Feb-96  Ted S.      Added return values so you know when 
                                    you have hit the end of the views.
                                    Without that you circle forever.

---------------------------------------------------------------------------*/

package pj.awt;

import pj.awt.Divider;
import pj.awt.Page;

import java.lang.String;

import collections.ImplementationError;

/**
 * A view onto one or more sections of a Personal Journal paper.
 * <P>
 * Views are maintained in a sequence.  The sequence definition is
 * implementation dependent.
  * <P>
 * A PaperView has a cursor into its sequence of views.  The cursor is
 * called the current view.  The current view is the view which the user
 * sees if an implementation instance is showing.  Moving the current view
 * cursor changes the view that a user sees.  The current view cursor may
 * be moved sequentially through a view stack by first, next, last, and
 * previous operations. The current view cursor may be moved to specific
 * view by the setCurrentPage operation.
 *
 * @version 0.00 07-Jan-96
 * @author  rphall
*/
public interface PaperView
    {

    // --- Public operations

    /**
     * @return The number of views in an implementation instance.
    */
    public int countViews();
 
    

    /**
     * Make the first view the current view.
    */
    public void firstView();

    /**
     * Make the next view (after the current view) the current view.
     *  a false return means you have hit the end.
    */
    public boolean nextView();

    /**
     * Make the last view the current view.
    */
    public void lastView();

    /**
     * Make the previous view (before current view) the current view.
     *  a false return means you have hit the end.
    */
    public boolean previousView();

    
     
    /**
     * Make a specific view the current view.
     * @param idx The index of the view to display.
    */
    public void view(int idx);

    } // PaperView
