/*---------------------------------------------------------------------------

	Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    Tab.java

        Attributes of a general Tab component.


	Authors:

        rphall          Rick Hall


	Version Ident:

        $Header$


	History:

        10-dec-1995 rphalll     Initial Creation


---------------------------------------------------------------------------*/

package pj.awt;

import pj.util.Mappable;
import java.lang.Object;
import java.lang.String;

/**
 * General attributes of a TabBar component. A tab is nameable;
 * the name of tab is used by TabBar to distinguish it from other
 * tabs.  A tab is mappable; the mapped object of a tab is a String
 * that holds a page name of a divider.  A tab also maintains a
 * a visual presentation object, which may be set or retrieved.
 *
 * @see pj.awt.TabBar
 * @see pj.util.Mappable
 * @see pj.awt.TabSpec
 * @version 0.00 03-Jan-96
 * @author Rick Hall
*/
public interface Tab extends Mappable
    {

    // --- Public operations

    /**
     * Get a tab's name.
     * Equivalent to getName().
     *
     * @return The internal id of a tab within a TabBar.
     * @see pj.util.Nameable#getName
    */
    public String getTabName();

    /**
     * Get the mapped name of a tab.
     * Equivalent to ((String)getMappedObject()).
     *
     * @return The name of the page which this tab references.
     * @see pj.util.Mappable#getMappedName
    */
    public String getPageName();

    /**
     * @param visual The user-visible presentation of a tab, typically
     * a String or an Image.
    */
    public void setTabVisual(Object visual);

    /**
     * @return The visual presentation of a tab, typically
     * a String or an Image.
    */
    public Object getTabVisual();

    } // Tab
