/*---------------------------------------------------------------------------

	Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)RemoteImage.java  0.00 16-Jan-96

        A RemoteImage is an image stored at a URL.

	Authors:

        rphall          Rick Hall

	Version Ident:

        $Header$

	History:

        16-Jan-1996 rphall     Initial Creation

---------------------------------------------------------------------------*/

package pj.net;

import pj.lang.EmptySecurityManager;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;

/**
 * A RemoteImage is an image stored at a URL.  The current implementation
 * assumes the image is in GIF format.
 * <P>
 * This class is totally unrelated to RemoteObject and subclasses
 * defined in the sun.tools.debug package.
 * <P>
 * This class is related to RemoteString and RemoteURL defined
 * in this package (pj.net).
 *
 * @see pj.net.RemoteString
 * @see pj.net.RemoteURL
 * @version 0.00 16-Jan-1996
 * @author Rick Hall
*/
public class RemoteImage
    {


    // --- Instance variables
    /**
     * The URL from which an image is loaded.
    */
    private URL url;

    /**
     * An image resolved from the url.
     * May be null if the url or its contents are invalid.
    */
    private Image img;


    // --- Public constructors

    /**
     * Constructs an RemoteImage based on the specified URL.
    */
    public RemoteImage(URL u)
        {
        url = u;
        img = null;
        }

    // --- Public operations

    /**
     * @return The source URL of this RemoteImage.
    */
    public URL getSourceURL()
        { 
        return url; 
        }

    /**
     * Get an image resolved from a url.
     * <P>
     * This operation does not automatically force a new resolve operation.
     * If the current image is not null, the current image is returned.
     * If the current image is null, this operation invokes resolveImage().
     * The returned image may still be null if the image can not be resolved
     * from the url.
     * @return The current image, or null if the current url or its contents
     * are invalid.
    */
    public Image getImage()
        {
        if (img == null)
            img = resolveImage();

        return img;
        }

    // --- Private operations

    /**
     * Resolve an image from the current url.
     * The image data is not loaded until it is needed.
     * @return May be null if url or its contents are invalid;
    */
    private Image resolveImage()
        {
        img = null;
        try {
            img = Toolkit.getDefaultToolkit().getImage(url);
            }
        catch(NullPointerException e)
            {
            try {
                // Due to a bug(?) in the beta2 JDK, an empty security manager
                // must be installed for components that are running in
                // standalone applications (as opposed to applets under browsers).
                EmptySecurityManager.install();
                img = Toolkit.getDefaultToolkit().getImage(url);
                }
            catch(SecurityException se)
                {
                System.out.println("Debug RemoteImage getImage, exception2 = " + se.toString() );
                } // catch se

            } // catch e1

        return img;
        }

    

    } // RemoteImage
